@@ -23,9 +23,12 @@ using System.IO;
 using System.Linq;
 using System.Runtime.Serialization;
 using System.Runtime.Serialization.Formatters.Binary;
+using System.Security;
+using System.Security.Permissions;
 
 using Do;
 using Do.Interface;
+using Do.Interface.Wink;
 using Do.Universe;
 using Do.Platform;
 
@@ -38,6 +41,8 @@ namespace Docky.Core.Default
 {
 	public class ItemsService : IItemsService
 	{
+		const int UpdateDelay = 20;
+		
 		// stores items tied with their original identifier so we can remove them later
 		Dictionary<string, AbstractDockItem> custom_items;
 		
@@ -49,6 +54,10 @@ namespace Docky.Core.Default
 		
 		bool CustomItemsRead { get; set; }
 		
+		IEnumerable<AbstractDockItem> Docklets {
+			get { return DockServices.DockletService.ActiveDocklets.Cast<AbstractDockItem> (); }
+		}
+		
 		//// <value>
 		/// Our main menu.  We only ever need one so we will store it here
 		/// </value>
@@ -77,6 +86,12 @@ namespace Docky.Core.Default
 			}
 		}
 		
+		string DesktopFilesDirectory {
+			get {
+				return Path.Combine (Services.Paths.UserDataDirectory, GetType ().Name + "_DesktopFiles");
+			}
+		}
+		
 		IEnumerable<AbstractDockItem> OrderedItems {
 			get {
 				return stat_items
@@ -110,6 +125,9 @@ namespace Docky.Core.Default
 			Separator = new SeparatorItem ();
 			MenuItem = new DoDockItem ();
 			
+			if (!Directory.Exists (DesktopFilesDirectory))
+				Directory.CreateDirectory (DesktopFilesDirectory);
+			
 			RegisterEvents ();
 		}
 		
@@ -166,6 +184,12 @@ namespace Docky.Core.Default
 			}
 		}
 		
+		void ResetDockelts ()
+		{
+			UnregisterDocklets ();
+			RegisterDocklets ();
+		}
+		
 		void RegisterDockItem (AbstractDockItem dockItem)
 		{
 			dockItem.UpdateNeeded += HandleUpdateNeeded;
@@ -180,41 +204,43 @@ namespace Docky.Core.Default
 				(dockItem as IRightClickable).RemoveClicked -= HandleRemoveClicked;
 		}
 
-		void HandleAppletVisibilityChanged(object sender, EventArgs e)
+		void HandleAppletVisibilityChanged (object sender, EventArgs e)
 		{
+			ResetDockelts ();
 			OnDockItemsChanged ();
 		}
 
-		void HandleAutomaticIconsChanged()
+		void HandleAutomaticIconsChanged ()
 		{
 			UpdateItems ();
 		}
 		
-		void HandleRemoveClicked(object sender, EventArgs e)
+		void HandleRemoveClicked (object sender, EventArgs e)
 		{
 			if (sender is AbstractDockItem)
 				RemoveItem (sender as AbstractDockItem);
 		}
 
-		void HandleWindowOpened(object o, WindowOpenedArgs args)
+		void HandleWindowOpened (object o, WindowOpenedArgs args)
 		{
-			if (!args.Window.IsSkipTasklist)
-				UpdateItems ();
+			// we do a delayed update so that we allow a small gap for wnck to catch up
+			if (!args.Window.IsSkipTasklist || args.Window.Pid != System.Diagnostics.Process.GetCurrentProcess ().Id)
+				DelayUpdateItems ();
 		}
 
-		void HandleWindowClosed(object o, WindowClosedArgs args)
+		void HandleWindowClosed (object o, WindowClosedArgs args)
 		{
-			if (!args.Window.IsSkipTasklist)
-				UpdateItems ();
+			if (!args.Window.IsSkipTasklist || args.Window.Pid != System.Diagnostics.Process.GetCurrentProcess ().Id)
+				DelayUpdateItems ();
 		}
 
-		void HandleUniverseInitialized(object sender, EventArgs e)
+		void HandleUniverseInitialized (object sender, EventArgs e)
 		{
 			UpdatesEnabled = true;
 			UpdateItems ();
 		}
 		
-		void HandleUpdateNeeded(object sender, UpdateRequestArgs args)
+		void HandleUpdateNeeded (object sender, UpdateRequestArgs args)
 		{
 			if (!DockItems.Contains (args.Item))
 				return;
@@ -225,7 +251,7 @@ namespace Docky.Core.Default
 		#endregion
 		
 		#region Item Updating
-		AbstractDockItem MaybeCreateCustomItem (string identifier)
+		AbstractDockItem MaybeCreateCustomItem (ref string identifier)
 		{
 			ItemDockItem customItem = null;
 			
@@ -234,6 +260,25 @@ namespace Docky.Core.Default
 			
 			if (File.Exists (identifier) || Directory.Exists (identifier)) {
 				if (identifier.EndsWith (".desktop")) {
+					bool writeable = true;
+					try {
+						using (FileStream stream = File.OpenWrite (identifier)) {
+							;
+						}
+					} catch {
+						writeable = false;
+					}
+					if (writeable && Path.GetDirectoryName (identifier) != DesktopFilesDirectory) {
+						string newFile = Path.Combine (DesktopFilesDirectory, Path.GetFileName (identifier));
+						try {
+							Log<ItemsService>.Error ("Could not add custom item with id: {0}, File already exists", identifier);
+							File.Copy (identifier, newFile);
+						} catch {
+							
+							return null; 
+						}
+						identifier = newFile;
+					}
 					Item o = Services.UniverseFactory.NewApplicationItem (identifier) as Item;
 					customItem = new ItemDockItem (o);
 				} else {
@@ -241,7 +286,7 @@ namespace Docky.Core.Default
 					customItem = new ItemDockItem (o);
 				}
 			} else {
-				Item e = Services.Core.GetElement (identifier) as Item;
+				Item e = Services.Core.GetItem (identifier);
 				if (e != null)
 					customItem = new ItemDockItem (e);
 				else
@@ -258,28 +303,46 @@ namespace Docky.Core.Default
 				item.Position = i++;
 		}
 		
+		void DelayUpdateItems ()
+		{
+			GLib.Timeout.Add (UpdateDelay, delegate {
+				UpdateItems ();
+				return false;
+			});
+		}
+		
 		void UpdateItems ()
 		{
 			if (!UpdatesEnabled)
 				return;
 			
-			UpdateStatItems ();
-			
 			if (!CustomItemsRead) {
 				foreach (string s in ReadCustomItems ())
 					InternalAddItemToDock (s, LastPosition + 1);
 				
+				UpdateStatItems ();
+				
 				Dictionary<string, int> sortDictionary = ReadSortDictionary ();
 				foreach (ItemDockItem item in OrderedItems.Where (di => di is ItemDockItem)) {
-					if (sortDictionary.ContainsKey (item.Element.UniqueId))
-						item.Position = sortDictionary [item.Element.UniqueId];
+					if (item.Item == null)
+						continue;
+					
+					if (sortDictionary.ContainsKey (item.Item.UniqueId))
+						item.Position = sortDictionary [item.Item.UniqueId];
+				}
+				foreach (AbstractDockItem item in Docklets) {
+					if (sortDictionary.ContainsKey (item.GetType ().FullName))
+					    item.Position = sortDictionary [item.GetType ().FullName];
 				}
 				
 				CustomItemsRead = true;
+			} else {
+				UpdateStatItems ();
 			}
 			
 			UpdateTaskItems ();
 			SimplifyPositions (OrderedItems);
+			SimplifyPositions (Docklets);
 			
 			OnDockItemsChanged ();
 		}
@@ -297,12 +360,12 @@ namespace Docky.Core.Default
 			IEnumerable<Item> mostUsedItems = MostUsedItems ();
 			
 			DateTime currentTime = DateTime.UtcNow;
-			foreach (Item item in mostUsedItems) {
-				if (local_cust.Any (di => di.Element == item))
-					continue;
+			foreach (Item item in mostUsedItems.Where (i => !local_cust.Any (ci => ci.Item != null && ci.Item.UniqueId == i.UniqueId))) {
 				
-				if (old_items.Any (di => di.Element == item)) {
-					stat_items.AddRange (old_items.Where (di => di.Element == item).Cast<AbstractDockItem> ());
+				if (old_items.Any (di => di.Item == item)) {
+					stat_items.AddRange (old_items
+					                     .Where (di => di.Item != null && di.Item.UniqueId == item.UniqueId)
+					                     .Cast<AbstractDockItem> ());
 				} else {
 					ItemDockItem di = new ItemDockItem (item);
 					RegisterDockItem (di);
@@ -310,7 +373,7 @@ namespace Docky.Core.Default
 					
 					int position = LastPosition + 1;
 
-					//TODO fixme once mono 1.9 support is dropped
+					// TODO fixme once mono 1.9 support is dropped
 					if (old_items.Any ())
 						position += old_items.Max ((Func<ItemDockItem, int>) (oi => oi.Position));
 
@@ -328,51 +391,64 @@ namespace Docky.Core.Default
 		
 		void UpdateTaskItems ()
 		{
-			foreach (ItemDockItem item in OrderedItems.Where (di => di is ItemDockItem)) {
+			foreach (ItemDockItem item in OrderedItems.Where (di => di is ItemDockItem))
 				item.UpdateApplication ();
-			}
 			
 			List<ApplicationDockItem> out_items = new List<ApplicationDockItem> ();
 
-			IEnumerable<int> knownPids = OrderedItems
+			IEnumerable<Window> knownWindows = OrderedItems
 				    .Where (di => di is ItemDockItem)
 					.Cast<ItemDockItem> ()
-					.SelectMany (di => di.Pids);
+					.SelectMany (di => di.Windows);
 			
-			IEnumerable<Application> prunedApps = WindowUtils.GetApplications ()
-				.Where (app => app.Windows.Any (w => !w.IsSkipTasklist))
-				.Where (app => !knownPids.Contains (app.Pid) && app.Windows.Any ());
+			var prunedWindows = WindowUtils.GetWindows ()
+				    .Where (w => !w.IsSkipTasklist && !knownWindows.Contains (w))
+					.GroupBy (w => SafeResClass (w));
 			
-			foreach (IEnumerable<Wnck.Application> apps in prunedApps
-			         .GroupBy (app => (app as Wnck.Application).Windows [0].ClassGroup.ResClass)) {
-				ApplicationDockItem api = new ApplicationDockItem (apps);
-
-				if (task_items.Any (di => di.Equals (api))) {
-					AbstractDockItem match = task_items.Where (di => di.Equals (api)).First ();
-					api.DockAddItem = match.DockAddItem;
-					api.Position = match.Position;
-				} else {
-					api.DockAddItem = DateTime.UtcNow;
-				}
-					
-				out_items.Add (api);
-				RegisterDockItem (api);
-			}
+			IEnumerable<ApplicationDockItem> apps = WindowUtils.GetWindows ()
+				    .Where (w => !w.IsSkipTasklist && !knownWindows.Contains (w))
+					.GroupBy (w => SafeResClass (w))
+					.Select (ws => CreateApplicationDockItem (ws, task_items));
+			
+			apps.ForEach (a => RegisterDockItem (a));
+			out_items.AddRange (apps);
 			
 			foreach (AbstractDockItem item in task_items) {
 				UnregisterDockItem (item);
 				item.Dispose ();
 			}
 			
-			foreach (ApplicationDockItem api in out_items.OrderBy (di => di.DockAddItem)) {
-				if (api.Position == 0)
-					api.Position = LastPosition + 1;
+			foreach (ApplicationDockItem adi in out_items.OrderBy (di => di.DockAddItem)) {
+				if (adi.Position == 0) // raw and unpositioned items
+					adi.Position = LastPosition + 1;
 			}
 					
 			task_items.Clear ();
 			task_items.AddRange (out_items.Cast<AbstractDockItem> ());
 		}
 		
+		ApplicationDockItem CreateApplicationDockItem (IEnumerable<Wnck.Window> windows, IEnumerable<AbstractDockItem> lastSet)
+		{
+			ApplicationDockItem adi = new ApplicationDockItem (windows);
+			
+			if (lastSet.Any (di => di.Equals (adi))) {
+				AbstractDockItem match = lastSet.Where (di => di.Equals (adi)).First ();
+				adi.DockAddItem = match.DockAddItem;
+				adi.Position = match.Position;
+				} else {
+				adi.DockAddItem = DateTime.UtcNow;
+			}
+			
+			return adi;
+		}
+		
+		string SafeResClass (Wnck.Window window)
+		{
+			if (window.ClassGroup != null && window.ClassGroup.ResClass != null)
+				return window.ClassGroup.ResClass;
+			return string.Empty;
+		}
+		
 		void OnDockItemsChanged ()
 		{
 			output_items.Clear ();
@@ -383,7 +459,7 @@ namespace Docky.Core.Default
 		#endregion
 		
 		#region Item Management
-		bool InternalAddItemToDock (Element item, int position)
+		bool InternalAddItemToDock (Item item, int position)
 		{
 			if (!(item is Item)) {
 				Log<ItemsService>.Error ("Could not add {0} to custom items for dock", item.Safe.Name);
@@ -409,7 +485,7 @@ namespace Docky.Core.Default
 		{
 			if (custom_items.ContainsKey (identifier)) return false;
 			
-			AbstractDockItem customItem = MaybeCreateCustomItem (identifier);
+			AbstractDockItem customItem = MaybeCreateCustomItem (ref identifier);
 			
 			if (customItem == null) return false;
 			
@@ -439,7 +515,13 @@ namespace Docky.Core.Default
 				
 				using (StreamWriter writer = new StreamWriter (SortDictionaryPath)) {
 					foreach (ItemDockItem di in OrderedItems.Where (di => di is ItemDockItem)) {
-						writer.WriteLine ("{0}|{1}", di.Element.UniqueId, di.Position);
+						if (di.Item == null)
+							continue;
+						
+						writer.WriteLine ("{0}|{1}", di.Item.UniqueId, di.Position);
+					}
+					foreach (AbstractDockItem di in Docklets) {
+						writer.WriteLine ("{0}|{1}", di.GetType ().FullName, di.Position);
 					}
 				}
 			} catch (Exception e) {
@@ -502,16 +584,7 @@ namespace Docky.Core.Default
 		#region Random Useful Functions
 		bool ItemCanInteractWithPosition (AbstractDockItem item, int position)
 		{
-			return DockItems.Contains (item) && 0 <= position && position < DockItems.Count;
-		}
-		
-		int LastNonTaskItemPosition ()
-		{
-			if (!OrderedItems.Any (di => !(di is ApplicationDockItem)))
-				return 0;
-			return OrderedItems
-				.Where (di => !(di is ApplicationDockItem))
-				.Max ((Func<AbstractDockItem, int>) (di => di.Position));
+			return (DockItems.Contains (item) || Docklets.Contains (item)) && position >= 0 && position < DockItems.Count; 
 		}
 		
 		/// <summary>
@@ -560,14 +633,14 @@ namespace Docky.Core.Default
 					// add a separator and any docklets that are active
 					if (DockServices.DockletService.ActiveDocklets.Any ()) {
 						output_items.Add (Separator);
-						output_items.AddRange (DockServices.DockletService.ActiveDocklets.Cast<AbstractDockItem> ());
+						output_items.AddRange (Docklets.OrderBy (docklet => docklet.Position));
 					}
 				}
 				return readonly_output_items;
 			}
 		}
 		
-		public void AddItemToDock (Element item)
+		public void AddItemToDock (Item item)
 		{
 			AddItemToDock (item, LastPosition + 1);
 		}
@@ -577,7 +650,7 @@ namespace Docky.Core.Default
 			AddItemToDock (identifier, LastPosition + 1);
 		}
 		
-		public void AddItemToDock (Element item, int position)
+		public void AddItemToDock (Item item, int position)
 		{
 			position = DockItems [position].Position;
 			if (InternalAddItemToDock (item, position)) {
@@ -602,7 +675,14 @@ namespace Docky.Core.Default
 			if (item < 0 || item > DockItems.Count)
 				return false;
 			
-			return (OrderedItems.Contains (DockItems [item]));
+			return (OrderedItems.Contains (DockItems [item]) || Docklets.Contains (DockItems [item]));
+		}
+		
+		public bool ItemCanBeRemoved (int item)
+		{
+			AbstractDockItem adi = DockItems [item];
+			return adi.WindowCount == 0 && ((GetIconSource (adi) == IconSource.Statistics && adi is ItemDockItem) ||
+			                                (GetIconSource (adi) == IconSource.Custom));
 		}
 		
 		public void DropItemOnPosition (AbstractDockItem item, int position)
@@ -615,19 +695,22 @@ namespace Docky.Core.Default
 					continue;
 				}
 				
-				if (item is ApplicationDockItem && position <= LastNonTaskItemPosition ()) {
-					ApplicationDockItem api = item as ApplicationDockItem;
-					string desktop_file = api.DesktopFile;
+				if (item is ApplicationDockItem) {
+					ApplicationDockItem adi = item as ApplicationDockItem;
 					
-					if (string.IsNullOrEmpty (desktop_file)) continue;
+					if (adi.Launcher == null) continue;
 					
-					AbstractDockItem newItem = MaybeCreateCustomItem (desktop_file);
+					Item launcher = adi.Launcher as Item;
+					if (launcher == null)
+						continue;
 					
-					if (newItem == null) continue;
+					AbstractDockItem newItem = new ItemDockItem (launcher);
 					
 					newItem.Position = item.Position;
 					newItem.DockAddItem = item.DockAddItem;
-					custom_items [desktop_file] = newItem;
+					custom_items [launcher.UniqueId] = newItem;
+					
+					RegisterDockItem (newItem);
 					UpdateItems ();
 					WriteData ();
 				}
@@ -652,7 +735,10 @@ namespace Docky.Core.Default
 			IconSource itemSource = GetIconSource (DockItems [item]);
 			IconSource targetSource = GetIconSource (DockItems [position]);
 			
-			if (itemSource == IconSource.Unknown || targetSource == IconSource.Unknown)
+			if (itemSource == IconSource.Unknown || 
+			    targetSource == IconSource.Unknown || 
+			    (itemSource == IconSource.Docklet && targetSource != IconSource.Docklet) ||
+			    (itemSource != IconSource.Docklet && targetSource == IconSource.Docklet))
 				return;
 			
 			AbstractDockItem primaryItem = DockItems [item];
@@ -661,7 +747,13 @@ namespace Docky.Core.Default
 			int startPosition = primaryItem.Position;
 			int targetPosition = targetItem.Position;
 			
-			foreach (AbstractDockItem di in OrderedItems) {
+			IEnumerable<AbstractDockItem> itemSet;
+			if (itemSource == IconSource.Docklet)
+				itemSet = Docklets;
+			else
+				itemSet = OrderedItems;
+			
+			foreach (AbstractDockItem di in itemSet) {
 				if (startPosition < targetPosition) {
 					// the item is being shifted to the right.  Everything greater than item up to and including target item
 					// needs to be shifted to the left
@@ -697,6 +789,9 @@ namespace Docky.Core.Default
 			if (custom_items.Values.Contains (item))
 				return IconSource.Custom;
 			
+			if (Docklets.Contains (item))
+				return IconSource.Docklet;
+			
 			return IconSource.Unknown;
 		}
 		
@@ -709,30 +804,34 @@ namespace Docky.Core.Default
 		
 		public bool RemoveItem (int item)
 		{
-			bool ret_val = false;
+			bool result = ItemCanBeRemoved (item);
 			
-			if (DockItems [item].WindowCount == 0) {
-				if (GetIconSource (DockItems [item]) == IconSource.Statistics && DockItems [item] is ItemDockItem) {
-					DockPreferences.AddBlacklistItem ((DockItems [item] as ItemDockItem).Element.UniqueId);
+			if (result) {
+				if (GetIconSource (DockItems [item]) == IconSource.Statistics) {
+					ItemDockItem di = (DockItems [item] as ItemDockItem);
+					DockPreferences.AddBlacklistItem ((DockItems [item] as ItemDockItem).Item.UniqueId);
 					DockPreferences.AutomaticIcons = Math.Max (0, DockPreferences.AutomaticIcons - 1);
-					UpdateItems ();
-					ret_val = true;
 				} else if (GetIconSource (DockItems [item]) == IconSource.Custom) {
-					foreach (KeyValuePair<string, AbstractDockItem> kvp in custom_items) {
-						if (kvp.Value.Equals (DockItems [item])) {
-							custom_items.Remove (kvp.Key);
-							
-							UpdateItems ();
-							ret_val = true;
-							break;
+					KeyValuePair<string, AbstractDockItem> kvp = custom_items
+						.Where (k => k.Value.Equals (DockItems [item]))
+						.First ();
+					
+					custom_items.Remove (kvp.Key);
+					
+					if (kvp.Key.EndsWith (".desktop") && Path.GetDirectoryName (kvp.Key) == DesktopFilesDirectory) {
+						try {
+							File.Delete (kvp.Key);
+						} catch {
+							// Don't really care, it was only a nice thing to do...
 						}
 					}
 				}
+				
+				UpdateItems ();
+				WriteData ();
 			}
 			
-			UpdateItems ();
-			WriteData ();
-			return ret_val;
+			return result;
 		}
 		
 		public bool HotSeatItem (AbstractDockItem item, List<AbstractDockItem> seatedItems)
