@@ -26,6 +26,8 @@ using Gdk;
 using Gtk;
 
 using Do.Platform;
+using Do.Interface.Wink;
+using Do.Interface.Xlib;
 
 using Docky.Core;
 using Docky.Utilities;
@@ -40,30 +42,45 @@ namespace Docky.Interface
 	{
 		public static readonly TimeSpan BaseAnimationTime = new TimeSpan (0, 0, 0, 0, 150);
 		
+		static DateTime UpdateTimeStamp (DateTime lastStamp, TimeSpan animationLength)
+		{
+			TimeSpan delta = DateTime.UtcNow - lastStamp;
+			if (delta < animationLength)
+				return DateTime.UtcNow.Subtract (animationLength - delta);
+			return DateTime.UtcNow;
+		}
+		
 		const uint OffDockWakeupTime = 250;
 		const uint OnDockWakeupTime = 20;
+		
+		const int UrgentBounceHeight = 80;
+		const int LaunchBounceHeight = 30;
 
 		TimeSpan BounceTime = new TimeSpan (0, 0, 0, 0, 700);
 		TimeSpan InsertAnimationTime = new TimeSpan (0, 0, 0, 0, 150*5);
 		
 		#region Private Variables
-		Gdk.Point cursor;
-		
 		DateTime enter_time = new DateTime (0);
 		DateTime interface_change_time = new DateTime (0);
 		DateTime last_draw_timeout = new DateTime (0);
-		DateTime cursor_update = new DateTime (0);
+		DateTime showhide_time = new DateTime (0);
+		DateTime painter_time = new DateTime (0);
 		
-		bool disposed;
+		TimeSpan painter_span;
 		
 		uint animation_timer;
-		uint cursor_timer;
 		
 		DockWindow window;
 		
+		DrawingService drawing_service;
+		
 		#endregion
 		
 		#region Public Properties
+		//// <value>
+		/// Returns true if the cursor is over the visible part of the dock or the pixel closest to its autohide edge
+		/// </value>
+		public bool CursorIsOverDockArea { get; private set; }
 		
 		/// <value>
 		/// The width of the docks window, but not the visible dock
@@ -75,6 +92,11 @@ namespace Docky.Interface
 		/// </value>
 		public int Height { get; private set; }
 		
+		//// <value>
+		/// Returns true if the dock is currently being overlayed by an IDockPainter
+		/// </value>
+		public bool PainterOverlayVisible { get; private set; }
+		
 		/// <value>
 		/// The width of the visible dock
 		/// </value>
@@ -88,51 +110,69 @@ namespace Docky.Interface
 		public int DockHeight {
 			get { return PositionProvider.DockHeight; }
 		}
-
+		
 		public uint[] StrutRequest {
 			get {
 				uint[] values = new uint[12];
-				Gdk.Rectangle geo = LayoutUtils.MonitorGemonetry ();
+				Gdk.Rectangle geo = LayoutUtils.MonitorGeometry ();
 				
-				if (DockPreferences.AutoHide || DockPreferences.AllowOverlap)
+				if (DockPreferences.AutohideType != AutohideType.None)
 					return values;
 				
 				switch (DockPreferences.Orientation) {
 				case DockOrientation.Bottom:
-					values [(int) XLib.Struts.Bottom] = (uint) (DockHeight + (Screen.Height - (geo.Y + geo.Height)));
-					values [(int) XLib.Struts.BottomStart] = (uint) geo.X;
-					values [(int) XLib.Struts.BottomEnd] = (uint) (geo.X + geo.Width - 1);
+					values [(int) Struts.Bottom] = (uint) (DockHeight + (Screen.Height - (geo.Y + geo.Height)));
+					values [(int) Struts.BottomStart] = (uint) geo.X;
+					values [(int) Struts.BottomEnd] = (uint) (geo.X + geo.Width - 1);
 					break;
 				case DockOrientation.Top:
-					values [(int) XLib.Struts.Top] = (uint) (DockHeight + geo.Y);
-					values [(int) XLib.Struts.TopStart] = (uint) geo.X;
-					values [(int) XLib.Struts.TopEnd] = (uint) (geo.X + geo.Width - 1);
+					values [(int) Struts.Top] = (uint) (DockHeight + geo.Y);
+					values [(int) Struts.TopStart] = (uint) geo.X;
+					values [(int) Struts.TopEnd] = (uint) (geo.X + geo.Width - 1);
 					break;
 				}
 				return values;
 			}
 		}
 
+		/// <value>
+		/// The current cursor as known to the dock.
+		/// </value>
+		public Gdk.Point Cursor {
+			get {
+				return CursorTracker.Cursor;
+			}
+		}
+		
+		public Gdk.Rectangle MinimumDockArea {
+			get {
+				return PositionProvider.MinimumDockArea;
+			}
+		}
 		#endregion
 		
+		AutohideTracker AutohideTracker { get; set; }
+		
+		CursorTracker CursorTracker { get; set; }
+		
+		DnDTracker DnDTracker { get; set; }
+		
 		DockAnimationState AnimationState { get; set; }
 		
 		ItemPositionProvider PositionProvider { get; set; }
 
 		new DockItemMenu PopupMenu { get; set; }
 		
-		bool CursorIsOverDockArea {	get; set; }
-
-		ModifierType CursorModifier { get; set; }
-		
 		ReadOnlyCollection<AbstractDockItem> DockItems { 
 			get { return DockServices.ItemsService.DockItems; } 
 		}
 		
 		AbstractDockItem CurrentDockItem {
 			get {
-				try { return DockItems [PositionProvider.IndexAtPosition (Cursor)]; }
-				catch { return null; }
+				int index = PositionProvider.IndexAtPosition (Cursor);
+				if (index >= 0 && index < DockItems.Count)
+					return DockItems [index];
+				return null;
 			}
 		}
 		
@@ -140,94 +180,56 @@ namespace Docky.Interface
 			get { return DockPreferences.SummonTime; }
 		}
 		
-		/// <value>
-		/// The current cursor as known to the dock.
-		/// </value>
-		public Gdk.Point Cursor {
-			get {
-				return cursor;
-			}
-			set {
-				bool cursorIsOverDockArea = CursorIsOverDockArea;
-				cursor = value;
-
-				// We set this value here instead of dynamically checking due to performance constraints.
-				// Ideally our CursorIsOverDockArea getter would do this fairly simple calculation, but it gets
-				// called about 20 to 30 times per render loop, so the savings do add up.
-				Gdk.Rectangle dockRegion;
-				if (PainterOverlayVisible)
-					dockRegion = GetDockArea ();
-				else
-					dockRegion = MinimumDockArea;
-				
-				if (cursorIsOverDockArea) {
-					dockRegion.Inflate (0, (int) (IconSize * (DockPreferences.ZoomPercent - 1)) + 22);
-					CursorIsOverDockArea = dockRegion.Contains (cursor);
-				} else {
-					if (DockPreferences.AutoHide) {
-						switch (DockPreferences.Orientation) {
-						case DockOrientation.Bottom:
-							dockRegion.Y += dockRegion.Height - 1;
-							dockRegion.Height = 1;
-							break;
-						case DockOrientation.Top:
-							dockRegion.Height = 1;
-							break;
-						}
-					}
-					CursorIsOverDockArea = dockRegion.Contains (cursor);
-				}
-				
-				// When we change over this boundry, it will normally trigger an animation, we need to be sure to catch it
-				if (CursorIsOverDockArea != cursorIsOverDockArea) {
-					ResetCursorTimer ();
-					enter_time = DateTime.UtcNow;
-					AnimatedDraw ();
-				}
-
-				DragCursorUpdate ();
-			}
+		bool WindowIntersectingOther { 
+			get { return AutohideTracker.WindowIntersectingOther; }
 		}
 		
-		Gdk.Rectangle MinimumDockArea {
+		IEnumerable<Gdk.Window> WindowStack {
 			get {
-				return PositionProvider.MinimumDockArea;
+				try {
+					return Screen.WindowStack;
+				} catch { 
+					try {
+						return Wnck.Screen.Default.WindowsStacked.Select (wnk => Gdk.Window.ForeignNew ((uint) wnk.Xid));
+					} catch {
+						return null;
+					}
+				}
 			}
 		}
 		
 		public DockArea (DockWindow window) : base ()
 		{
+			drawing_service = new DrawingService (this);
+			DockServices.RegisterService (drawing_service);
+			
 			this.window = window;
 			
-			SetSize ();
+			ScreenUtils.Initialize ();
+			WindowUtils.Initialize ();
 			
+			AnimationState   = new DockAnimationState ();
+			AutohideTracker  = new AutohideTracker (this);
+			CursorTracker    = new CursorTracker (window, OffDockWakeupTime);
 			PositionProvider = new ItemPositionProvider (this);
+			DnDTracker       = new DnDTracker (this, PositionProvider, CursorTracker);
+			PopupMenu        = new DockItemMenu ();
 			
-			AnimationState = new DockAnimationState ();
 			BuildAnimationStateEngine ();
-			
-			PopupMenu = new DockItemMenu ();
-
-			Cursor = new Gdk.Point (-1, -1);
 
 			this.SetCompositeColormap ();
 
-			// fixme, we should be using the PointerMotionHintMask
-			AddEvents ((int) EventMask.PointerMotionMask |
-			           (int) EventMask.EnterNotifyMask |
-			           (int) EventMask.ButtonPressMask | 
+			AddEvents ((int) EventMask.ButtonPressMask | 
 			           (int) EventMask.ButtonReleaseMask |
 			           (int) EventMask.ScrollMask |
 			           (int) EventMask.FocusChangeMask);
 			
+			SetSize ();
 			DoubleBuffered = false;
 
 			BuildRendering ();
-			BuildDragAndDrop ();
 			
 			RegisterEvents ();
-			RegisterGtkDragDest ();
-			RegisterGtkDragSource ();
 			
 			ResetCursorTimer ();
 		}
@@ -235,10 +237,11 @@ namespace Docky.Interface
 		void SetSize ()
 		{
 			Gdk.Rectangle geo;
-			geo = LayoutUtils.MonitorGemonetry ();
+			geo = LayoutUtils.MonitorGeometry ();
 			
 			Width = geo.Width;
-			Height = 300;
+			Height = DockPreferences.FullIconSize + 2 * PositionProvider.VerticalBuffer + UrgentBounceHeight;
+			Height = Math.Max (150, Height);
 			
 			SetSizeRequest (Width, Height);
 		}
@@ -250,6 +253,7 @@ namespace Docky.Interface
 			
 			DockPreferences.MonitorChanged += HandleMonitorChanged; 
 			DockPreferences.IconSizeChanged += HandleIconSizeChanged; 
+			DockPreferences.OrientationChanged += HandleOrientationChanged; 
 			
 			DockServices.PainterService.PainterShowRequest += HandlePainterShowRequest;
 			DockServices.PainterService.PainterHideRequest += HandlePainterHideRequest;
@@ -264,6 +268,14 @@ namespace Docky.Interface
 			Realized += (o, e) => SetParentInputMask ();
 			Realized += (o, a) => GdkWindow.SetBackPixmap (null, false);
 			
+			AutohideTracker.IntersectionChanged += HandleIntersectionChanged;
+			Wnck.Screen.Default.ActiveWindowChanged += HandleActiveWindowChanged;
+			
+			CursorTracker.CursorUpdated += HandleCursorUpdated;
+			
+			DnDTracker.DragEnded += HandleDragEnded;
+			DnDTracker.DrawRequired += HandleDrawRequired; 
+			
 			StyleSet += (o, a) => { 
 				if (IsRealized)
 					GdkWindow.SetBackPixmap (null, false);
@@ -277,6 +289,7 @@ namespace Docky.Interface
 			
 			DockPreferences.MonitorChanged -= HandleMonitorChanged;
 			DockPreferences.IconSizeChanged -= HandleIconSizeChanged; 
+			DockPreferences.OrientationChanged -= HandleOrientationChanged; 
 			
 			DockServices.PainterService.PainterShowRequest -= HandlePainterShowRequest;
 			DockServices.PainterService.PainterHideRequest -= HandlePainterHideRequest;
@@ -287,6 +300,14 @@ namespace Docky.Interface
 			PopupMenu.Shown -= OnDockItemMenuShown;
 			
 			Services.Core.UniverseInitialized -= HandleUniverseInitialized;
+			
+			AutohideTracker.IntersectionChanged -= HandleIntersectionChanged;
+			Wnck.Screen.Default.ActiveWindowChanged -= HandleActiveWindowChanged;
+			
+			CursorTracker.CursorUpdated -= HandleCursorUpdated;
+			
+			DnDTracker.DragEnded -= HandleDragEnded;
+			DnDTracker.DrawRequired -= HandleDrawRequired; 
 		}
 		
 		void BuildAnimationStateEngine ()
@@ -298,8 +319,8 @@ namespace Docky.Interface
 			                             () => (CursorIsOverDockArea && ZoomIn != 1) || (!CursorIsOverDockArea && ZoomIn != 0));
 			
 			AnimationState.AddCondition (Animations.Open,
-			                             () => DateTime.UtcNow - enter_time < SummonTime ||
-			                             DateTime.UtcNow - interface_change_time < SummonTime);
+			                             () => DateTime.UtcNow - enter_time < BaseAnimationTime ||
+			                             DateTime.UtcNow - interface_change_time < BaseAnimationTime);
 			
 			AnimationState.AddCondition (Animations.Bounce,
 			                             () => DockItems.Any (di => di.TimeSinceClick <= BounceTime));
@@ -308,13 +329,50 @@ namespace Docky.Interface
 			                             () => DockItems.Any (di => DateTime.UtcNow - di.AttentionRequestStartTime < BounceTime));
 			
 			AnimationState.AddCondition (Animations.InputModeChanged,
-			                             () => DateTime.UtcNow - interface_change_time < SummonTime);
+			                             () => DateTime.UtcNow - interface_change_time < SummonTime || 
+			                             DateTime.UtcNow - showhide_time < SummonTime);
+		}
+		
+		void HandleCursorUpdated (object sender, CursorUpdatedArgs args)
+		{
+			if (PopupMenu.Visible)
+				return;
+			
+			UpdateCursorIsOverDockArea ();
+
+			bool cursorMoveWarrantsDraw = CursorIsOverDockArea && args.OldCursor.X != args.NewCursor.X;
+
+			if (DnDTracker.DragResizing || cursorMoveWarrantsDraw) 
+				AnimatedDraw ();
+		}
+		
+		void HandleDrawRequired(object sender, EventArgs e)
+		{
+			AnimatedDraw ();
+		}
+
+		void HandleDragEnded(object sender, EventArgs e)
+		{
+			ResetCursorTimer ();
+			Reconfigure ();
 		}
 
 		void HandleItemNeedsUpdate (object sender, UpdateRequestArgs args)
 		{
 			if (args.Type == UpdateRequestType.NeedsAttentionSet) {
 				SetParentInputMask ();
+				Reconfigure ();
+				
+				GLib.Timeout.Add ((uint) BounceTime.Milliseconds + 20, delegate {
+					Reconfigure ();
+					return false;
+				});
+			}
+			
+			if (args.Type == UpdateRequestType.IconChanged || args.Type == UpdateRequestType.NameChanged) {
+				RequestIconRender (args.Item);
+			} else {
+				RequestFullRender ();
 			}
 			AnimatedDraw ();
 		}
@@ -323,23 +381,33 @@ namespace Docky.Interface
 		{
 			Reconfigure ();
 		}
+
+		void HandleOrientationChanged()
+		{
+			Reconfigure ();
+			window.Reposition ();
+		}
 		
-		void HandleSizeChanged(object sender, EventArgs e)
+		void HandleSizeChanged (object sender, EventArgs e)
 		{
 			Reconfigure ();
 		}
 		
-		void Reconfigure ()
+		public void Reconfigure ()
 		{
+			if (!Visible || !IsRealized || DnDTracker.DragResizing)
+				return;
+			
 			SetSize ();
 			ResetBuffers ();
 			PositionProvider.ForceUpdate ();
 			SetParentInputMask ();
 			SetIconRegions ();
 			window.DelaySetStruts ();
+			AnimatedDraw ();
 		}
 		
-		void HandleUniverseInitialized(object sender, EventArgs e)
+		void HandleUniverseInitialized (object sender, EventArgs e)
 		{
 			GLib.Timeout.Add (2000, delegate {
 				DockServices.ItemsService.ForceUpdate ();
@@ -350,6 +418,7 @@ namespace Docky.Interface
 		
 		void HandleIconSizeChanged()
 		{
+			Reconfigure ();
 			AnimatedDraw ();
 		}
 
@@ -358,12 +427,10 @@ namespace Docky.Interface
 			if (sender != Painter && sender != LastPainter) return;
 			
 			if (args.Animated) {
-				if (AnimationState.Contains (Animations.Painter))
-					AnimationState.RemoveCondition (Animations.Painter);
-				TimeSpan span = args.AnimationLength;
-				
-				DateTime current_time = DateTime.UtcNow;
-				AnimationState.AddCondition (Animations.Painter, () => DateTime.UtcNow - current_time < span);
+				painter_span = args.AnimationLength;
+				painter_time = DateTime.UtcNow;
+				if (!AnimationState.Contains (Animations.Painter))
+					AnimationState.AddCondition (Animations.Painter, () => DateTime.UtcNow - painter_time < painter_span);
 			}
 			AnimatedDraw ();
 		}
@@ -376,7 +443,7 @@ namespace Docky.Interface
 
 			Painter = null;
 			PainterOverlayVisible = false;
-			interface_change_time = DateTime.UtcNow;
+			interface_change_time = UpdateTimeStamp (interface_change_time, SummonTime);
 
 			SetParentInputMask ();
 			AnimatedDraw ();
@@ -387,7 +454,7 @@ namespace Docky.Interface
 			});
 
 			window.UnpresentWindow ();
-			RegisterGtkDragSource ();
+			DnDTracker.Enable ();
 		}
 
 		void HandlePainterShowRequest(object sender, EventArgs e)
@@ -401,7 +468,7 @@ namespace Docky.Interface
 					Painter.Interrupt ();
 				Painter = painter;
 				PainterOverlayVisible = true;
-				interface_change_time = DateTime.UtcNow;
+				interface_change_time = UpdateTimeStamp (interface_change_time, SummonTime);
 
 				SetParentInputMask ();
 				AnimatedDraw ();
@@ -410,29 +477,69 @@ namespace Docky.Interface
 			}
 			
 			window.PresentWindow ();
-			UnregisterGtkDragSource ();
+			DnDTracker.Disable ();
 		}
 		
 		void ResetCursorTimer ()
 		{
-			if (disposed)
-				return;
-			if (cursor_timer > 0)
-				GLib.Source.Remove (cursor_timer);
-			
-			uint time = (CursorIsOverDockArea || drag_resizing) ? OnDockWakeupTime : OffDockWakeupTime;
-			cursor_timer = GLib.Timeout.Add (time, OnCursorTimerEllapsed);
+			CursorTracker.TimerLength = (CursorIsOverDockArea || DnDTracker.DragResizing) ? OnDockWakeupTime : OffDockWakeupTime;
 		}
 		
-		bool OnCursorTimerEllapsed ()
+		void UpdateCursorIsOverDockArea ()
 		{
-			ManualCursorUpdate ();
+			bool tmp = CursorIsOverDockArea;
 			
-			// if we have a painter visible this takes care of interrupting it on mouse off
-			if (!CursorIsOverDockArea && PainterOverlayVisible && (DateTime.UtcNow - enter_time).TotalMilliseconds > 400)
-				InterruptPainter ();
+			Gdk.Rectangle dockRegion;
+			if (PainterOverlayVisible)
+				dockRegion = GetDockArea ();
+			else
+				dockRegion = MinimumDockArea;
+			
+			if (tmp) {
+				dockRegion.Inflate (0, (int) (IconSize * (DockPreferences.ZoomPercent - 1)) + 22);
+			} else {
+				if (IsHidden) {
+					switch (DockPreferences.Orientation) {
+					case DockOrientation.Bottom:
+						dockRegion.Y += dockRegion.Height - 1;
+						dockRegion.Height = 1;
+						break;
+					case DockOrientation.Top:
+						dockRegion.Height = 1;
+						break;
+					}
+				}
+			}
+			
+			if (PainterOverlayVisible)
+				CursorIsOverDockArea = dockRegion.Contains (new Gdk.Point (dockRegion.X, Cursor.Y));
+			else
+				CursorIsOverDockArea = dockRegion.Contains (Cursor);
+			
+			if (CursorIsOverDockArea != tmp) {
+				ResetCursorTimer ();
+				enter_time = UpdateTimeStamp (enter_time, BaseAnimationTime);
+				
+				switch (DockPreferences.AutohideType) {
+				case AutohideType.Autohide:
+					showhide_time = enter_time;
+					break;
+				case AutohideType.Intellihide:
+					if (WindowIntersectingOther)
+						showhide_time = enter_time;
+					break;
+				}
+				AnimatedDraw ();
+				
+				if (PainterOverlayVisible) {
+					GLib.Timeout.Add (500, delegate {
+						if (!CursorIsOverDockArea && PainterOverlayVisible && (DateTime.UtcNow - enter_time).TotalMilliseconds > 400)
+							InterruptPainter ();
+						return false;
+					});
+				}
+			}
 			
-			return true;
 		}
 		
 		void AnimatedDraw ()
@@ -487,6 +594,8 @@ namespace Docky.Interface
 			DockPreferences.MaxIconSize = (int) (((double) Width / MinimumDockArea.Width) * IconSize);
 			
 			SetIconRegions ();
+			RequestFullRender ();
+			UpdateCursorIsOverDockArea ();
 			AnimatedDraw ();
 		}
 		
@@ -495,7 +604,7 @@ namespace Docky.Interface
 			// While an popup menus are being showing, the dock does not recieve mouse updates.  This is
 			// both a good thing and a bad thing.  We must at the very least update the cursor position once the
 			// popup is no longer in view.
-			ManualCursorUpdate ();
+			CursorTracker.Enabled = true;
 			AnimatedDraw ();
 		}
 		
@@ -504,80 +613,10 @@ namespace Docky.Interface
 		/// </summary>
 		void OnDockItemMenuShown (object o, EventArgs args)
 		{
+			CursorTracker.Enabled = false;
 			AnimatedDraw ();
 		}
-		
-		void ManualCursorUpdate ()
-		{
-			if ((DateTime.UtcNow - cursor_update).TotalMilliseconds < 15) {
-				return;
-			}
-			
-			int x, y;
-			ModifierType mod;
-			Gdk.Screen screen;
-			
-			Display.GetPointer (out screen, out x, out y, out mod);
-			
-			if (screen == Screen) {
-				Gdk.Rectangle geo, hide_offset;
-				window.GetBufferedPosition (out geo.X, out geo.Y);
-				window.WindowHideOffset (out hide_offset.X, out hide_offset.Y);
 
-				x -= geo.X - hide_offset.X;
-				y -= geo.Y - hide_offset.Y;
-			} else {
-				x = -4000;
-				y = -4000;
-			}
-			
-			SetupCursor (x, y, mod);
-		}
-		
-		void SetupCursor (int x, int y, ModifierType mod)
-		{
-			CursorModifier = mod;
-			
-			if ((Cursor.X == x && Cursor.Y == y) || PopupMenu.Visible)
-				return;
-			
-			Gdk.Point old_cursor_location = Cursor;
-			Cursor = new Gdk.Point (x, y);
-
-			ConfigureCursor ();
-
-			if (drag_resizing)
-				HandleDragMotion ();
-			
-			bool cursorMoveWarrantsDraw = CursorIsOverDockArea && old_cursor_location.X != Cursor.X;
-
-			if (drag_resizing || cursorMoveWarrantsDraw) 
-				AnimatedDraw ();
-		}
-
-		protected override bool OnMotionNotifyEvent (Gdk.EventMotion evnt)
-		{
-			GtkDragging = false;
-			SetupCursor ((int) evnt.X, (int) evnt.Y, evnt.State);
-			cursor_update = DateTime.UtcNow;
-			return base.OnMotionNotifyEvent (evnt);
-		}
-		
-		protected override bool OnEnterNotifyEvent (Gdk.EventCrossing evnt)
-		{
-			ResetCursorTimer ();
-			ManualCursorUpdate ();
-			return base.OnEnterNotifyEvent (evnt);
-		}
-
-		protected override bool OnButtonPressEvent (Gdk.EventButton evnt)
-		{
-			if (CursorNearDraggableEdge)
-				StartDrag ();
-			
-			return base.OnButtonPressEvent (evnt);
-		}
-		
 		public void ProxyButtonReleaseEvent (Gdk.EventButton evnt)
 		{
 			HandleButtonReleaseEvent (evnt);
@@ -594,10 +633,7 @@ namespace Docky.Interface
 		private void HandleButtonReleaseEvent (Gdk.EventButton evnt)
 		{
 			// lets not do anything in this case
-			if (drag_resizing) {
-				EndDrag ();
-				return;
-			}
+			if (DnDTracker.DragResizing) return;
 			
 			if (PainterOverlayVisible) {
 				Painter.Clicked (GetDockArea (), Cursor);
@@ -630,37 +666,56 @@ namespace Docky.Interface
 				}
 				
 				//send off the clicks
-				Gdk.Point relative_point = Gdk.Point.Zero;
+				PointD relative_point = RelativePointOverItem (item);
 				DockItems [item].Clicked (evnt.Button, evnt.State, relative_point);
 				
 				AnimatedDraw ();
 			}
 			return;
 		}
+		
+		PointD RelativePointOverItem (int item)
+		{
+			PointD relative_point = new PointD (0,0);
+			double zoom;
+			PointD center;
+			IconZoomedPosition (item, out center, out zoom);
+			
+			int left = (int) (center.X - DockItems [item].Width * zoom / 2);
+			int top = (int) (center.Y - DockItems [item].Height * zoom / 2);
+			int right = (int) (center.X + DockItems [item].Width * zoom / 2);
+			int bottom = (int) (center.Y + DockItems [item].Height * zoom / 2);
+			
+			relative_point.X = (Cursor.X - left) / (double) (right - left);
+			relative_point.Y = (Cursor.Y - top) / (double) (bottom - top);
+			
+			return relative_point;
+		}
 
 		protected override bool OnScrollEvent (Gdk.EventScroll evnt)
 		{
-			int item = PositionProvider.IndexAtPosition ((int) evnt.X, (int) evnt.Y);
-			if (item == -1)
-				return false;
-			
-			DockItems [item].Scrolled (evnt.Direction);
+			if (PainterOverlayVisible) {
+				Painter.Scrolled (evnt.Direction);
+			} else {
+				int item = PositionProvider.IndexAtPosition ((int) evnt.X, (int) evnt.Y);
+				if (item >= 0 && item < DockItems.Count)
+					DockItems [item].Scrolled (evnt.Direction);
+			}
 			return base.OnScrollEvent (evnt);
 		}
 		
 		void SetIconRegions ()
 		{
-			Gdk.Rectangle pos, area, offset;
+			Gdk.Rectangle pos, area;
 			window.GetPosition (out pos.X, out pos.Y);
 			window.GetSize (out pos.Width, out pos.Height);
 			
-			window.WindowHideOffset (out offset.X, out offset.Y);
 			// we use geo here instead of our position for the Y value because we know the parent window
 			// may offset us when hidden. This is not desired...
 			for (int i = 0; i < DockItems.Count; i++) {
 				Gdk.Point position = PositionProvider.IconUnzoomedPosition (i);
-				area = new Gdk.Rectangle (pos.X + (position.X - IconSize / 2) - offset.X,
-				                          pos.Y + (position.Y - IconSize / 2) - offset.Y,
+				area = new Gdk.Rectangle (pos.X + (position.X - IconSize / 2),
+				                          pos.Y + (position.Y - IconSize / 2),
 				                          IconSize,
 				                          IconSize);
 				DockItems [i].SetIconRegion (area);
@@ -679,18 +734,14 @@ namespace Docky.Interface
 				offset = GetDockArea ().Height;
 				offset = offset * 2 + 10;
 			} else {
-				if (DockPreferences.AutoHide && !drag_resizing) {
-					// setting the offset to 2 will trigger the parent window to unhide us if we are hidden.
-					if (AnimationState [Animations.UrgencyChanged])
-						offset = 2;
-					else
-						offset = 1;
+				if (IsHidden && !DnDTracker.DragResizing) {
+					offset = 1;
 				} else {
 					offset = GetDockArea ().Height;
 				}
 			}
 			
-			int dockSize = (drag_resizing) ? Width : MinimumDockArea.Width;
+			int dockSize = (DnDTracker.DragResizing) ? Width : MinimumDockArea.Width;
 			
 			switch (DockPreferences.Orientation) {
 			case DockOrientation.Bottom:
@@ -717,22 +768,27 @@ namespace Docky.Interface
 			PainterOverlayVisible = false;
 			interface_change_time = DateTime.UtcNow;
 			
-			RegisterGtkDragSource ();
+			DnDTracker.Enable ();
 			window.UnpresentWindow ();
-
+			
 			SetParentInputMask ();
 			AnimatedDraw ();
 		}
 		
 		public override void Dispose ()
 		{
-			disposed = true;
+			DockServices.UnregisterService (drawing_service).Dispose ();
+			
 			UnregisterEvents ();
-			UnregisterGtkDragSource ();
+			DnDTracker.Disable ();
 
+			AutohideTracker.Dispose ();
+			CursorTracker.Dispose ();
+			DnDTracker.Dispose ();
 			PositionProvider.Dispose ();
 			AnimationState.Dispose ();
 			PopupMenu.Destroy ();
+			PopupMenu.Dispose ();
 
 			PositionProvider = null;
 			AnimationState = null;
@@ -757,9 +813,6 @@ namespace Docky.Interface
 			
 			window = null;
 			
-			if (cursor_timer > 0)
-				GLib.Source.Remove (cursor_timer);
-			
 			if (animation_timer > 0)
 				GLib.Source.Remove (animation_timer);
 
