@@ -23,9 +23,11 @@ using System.Linq;
 using Cairo;
 using Gdk;
 using Gtk;
+using Wnck;
 
 using Do.Interface;
 using Do.Interface.CairoUtils;
+using Do.Interface.Wink;
 
 using Docky.Core;
 using Docky.Utilities;
@@ -41,6 +43,12 @@ namespace Docky.Interface
 			public bool ForceFullRender { get; set; }
 			public Gdk.Point LastCursor { get; set; }
 			public double ZoomIn { get; set; }
+			public List<AbstractDockItem> RenderItems { get; private set; }
+			
+			public PreviousRenderData ()
+			{
+				RenderItems = new List<AbstractDockItem> ();
+			}
 		}
 		
 		public static DateTime RenderTime { get; private set; }
@@ -49,31 +57,68 @@ namespace Docky.Interface
 		const int UrgentIndicatorSize = 12;
 		
 		Dictionary<IDockPainter, Surface> painter_surfaces;
-		bool fast_render_fail, first_render_set;
+		bool next_fast_render, first_render_set, last_no_render, rendering;
+		double? zoom_in_buffer;
 		
 		Surface backbuffer, input_area_buffer, dock_icon_buffer;
 		Surface indicator, urgent_indicator;
 		IDockPainter painter, last_painter;
 		
+		DateTime LastOverlapCheck { get; set; }
+		
 		DateTime ActiveIconChangeTime { get; set; }
 		
 		DateTime FirstRenderTime { get; set; }
 		
 		PreviousRenderData RenderData { get; set; }
 
-		bool PainterOverlayVisible { get; set; }
-
+		//// <value>
+		/// Determines if the current rendering state allows for a fast render
+		/// </value>
 		bool CanFastRender {
 			get {
-				bool canFastRender = !RenderData.ForceFullRender && 
-					    RenderData.ZoomIn == 1 && 
-						ZoomIn == 1 && 
-						!AnimationState [Animations.IconInsert] &&
-						!AnimationState [Animations.UrgencyChanged] &&
-						!AnimationState [Animations.Bounce];
-				
-				fast_render_fail = canFastRender;
-				return canFastRender && !fast_render_fail;
+				bool result = next_fast_render && !RenderData.ForceFullRender && RenderData.RenderItems.Count == 0;
+				next_fast_render = RenderData.ZoomIn == 1 && ZoomIn == 1 && !DnDTracker.DragResizing;
+				return result;
+			}
+		}
+		
+		//// <value>
+		/// Determines if the current rendering state allows for a "no" render
+		/// </value>
+		bool CanNoRender {
+			get {
+				bool result = DockPreferences.ZoomEnabled && 
+					    !RenderData.ForceFullRender &&
+						RenderData.RenderItems.Count == 0 &&
+					    RenderData.ZoomIn == 0 &&
+						!DnDTracker.GtkDragging &&
+						!DnDTracker.DragResizing &&
+						ZoomIn == 0;
+				bool tmp = last_no_render;
+				last_no_render = result;
+				return result && tmp;
+			}
+		}
+		
+		//// <value>
+		/// Determins if a single item only needs to be redrawn and updated (usually caused by an update request)
+		/// </value>
+		bool SingleItemRender {
+			get {
+				return !RenderData.ForceFullRender && 
+					    RenderData.ZoomIn == 0 && 
+						ZoomIn == 0 && 
+						RenderData.RenderItems.Count == 1;
+			}
+		}
+		
+		bool AnimationRequiresRender {
+			get {
+				return AnimationState [Animations.IconInsert] ||
+					   AnimationState [Animations.UrgencyChanged] ||
+					   AnimationState [Animations.Bounce] ||
+					   AnimationState [Animations.InputModeChanged];
 			}
 		}
 		
@@ -97,6 +142,24 @@ namespace Docky.Interface
 			}
 		}
 		
+		//// <value>
+		/// Get autohide state
+		/// </value>
+		bool IsHidden {
+			get {
+				bool hidden = false;
+				switch (DockPreferences.AutohideType) {
+				case AutohideType.Autohide:
+					hidden = !CursorIsOverDockArea;
+					break;
+				case AutohideType.Intellihide:
+					hidden = !CursorIsOverDockArea && WindowIntersectingOther;
+					break;
+				}
+				return hidden;
+			}
+		}
+		
 		/// <value>
 		/// Icon Size used for the dock
 		/// </value>
@@ -122,7 +185,7 @@ namespace Docky.Interface
 				return painter;
 			}
 			set {
-				if (value == painter)
+				if (painter == value)
 					return;
 				LastPainter = painter;
 				painter = value;
@@ -137,7 +200,7 @@ namespace Docky.Interface
 		double PainterOpacity {
 			get { return 1 - DockIconOpacity; }
 		}
-
+		
 		//// <value>
 		/// The overall offset of the dock as a whole
 		/// </value>
@@ -145,7 +208,7 @@ namespace Docky.Interface
 			get {
 				double offset = 0;
 				// we never hide in these conditions
-				if (!DockPreferences.AutoHide || drag_resizing || PainterOpacity == 1) {
+				if (DockPreferences.AutohideType == AutohideType.None || DnDTracker.DragResizing || PainterOpacity == 1) {
 					if ((RenderTime - FirstRenderTime) > SummonTime)
 						return 0;
 					offset = 1 - Math.Min (1, (DateTime.UtcNow - FirstRenderTime).TotalMilliseconds / SummonTime.TotalMilliseconds);
@@ -153,10 +216,10 @@ namespace Docky.Interface
 				}
 
 				if (PainterOpacity > 0) {
-					if (CursorIsOverDockArea) {
+					if (!IsHidden) {
 						return 0;
 					} else {
-						offset = Math.Min (1, (RenderTime - enter_time).TotalMilliseconds / 
+						offset = Math.Min (1, (RenderTime - showhide_time).TotalMilliseconds / 
 						                   SummonTime.TotalMilliseconds);
 						offset = Math.Min (offset, Math.Min (1, 
 						                                     (RenderTime - interface_change_time)
@@ -166,9 +229,9 @@ namespace Docky.Interface
 					if (PainterOverlayVisible)
 						offset = 1 - offset;
 				} else {
-					offset = Math.Min (1, (RenderTime - enter_time).TotalMilliseconds / 
+					offset = Math.Min (1, (RenderTime - showhide_time).TotalMilliseconds / 
 					                   SummonTime.TotalMilliseconds);
-					if (CursorIsOverDockArea)
+					if (!IsHidden)
 						offset = 1 - offset;
 				}
 				return (int) (offset * PositionProvider.DockHeight * 1.5);
@@ -180,13 +243,20 @@ namespace Docky.Interface
 		/// </value>
 		double ZoomIn {
 			get {
-				if (drag_resizing && drag_start_point != Cursor)
+				if (DnDTracker.InternalDragActive)
 					return 0;
 				
+				// we buffer this value during renders since it will be checked many times and we dont need to 
+				// recalculate it each time
+				if (zoom_in_buffer.HasValue && rendering) {
+					return zoom_in_buffer.Value;
+				}
+				
 				double zoom = Math.Min (1, (RenderTime - enter_time).TotalMilliseconds / 
 					                 BaseAnimationTime.TotalMilliseconds);
 				if (CursorIsOverDockArea) {
-					if (DockPreferences.AutoHide)
+					if (DockPreferences.AutohideType == AutohideType.Autohide || 
+					    (DockPreferences.AutohideType == AutohideType.Intellihide && WindowIntersectingOther))
 						zoom = 1;
 				} else {
 					zoom = 1 - zoom;
@@ -195,6 +265,9 @@ namespace Docky.Interface
 				if (PainterOverlayVisible)
 					zoom = zoom * DockIconOpacity;
 				
+				if (rendering)
+					zoom_in_buffer = zoom;
+				
 				return zoom;
 			}
 		}
@@ -205,9 +278,25 @@ namespace Docky.Interface
 			painter_surfaces = new Dictionary<IDockPainter, Surface> ();
 		}
 		
-		void DrawDrock (Context cr)
+		void HandleActiveWindowChanged (object o, ActiveWindowChangedArgs args)
+		{
+			RequestFullRender ();
+			AnimatedDraw ();
+		}
+
+		void HandleIntersectionChanged (object sender, EventArgs e)
+		{
+			if (DockPreferences.AutohideType == AutohideType.Intellihide && !CursorIsOverDockArea) {
+				showhide_time = UpdateTimeStamp (showhide_time, SummonTime);
+				AnimatedDraw ();
+			}
+		}
+		
+		void DrawDock (Context cr)
 		{
 			Gdk.Rectangle dockArea = GetDockArea ();
+			window.SetBackgroundBlur (dockArea);
+			
 			DockBackgroundRenderer.RenderDockBackground (cr, dockArea);
 
 			IDockPainter dpaint = (Painter == null) ? LastPainter : Painter;
@@ -225,13 +314,13 @@ namespace Docky.Interface
 				cr.PaintWithAlpha (PainterOpacity);
 			}
 			
-			bool isNotSummonTransition = PainterOpacity == 0 || CursorIsOverDockArea || !DockPreferences.AutoHide;
+			bool isNotSummonTransition = PainterOpacity == 0 || !IsHidden || !DockPreferences.AutoHide;
 			if (DockIconOpacity > 0 && isNotSummonTransition) {
 				if (dock_icon_buffer == null)
 					dock_icon_buffer = cr.Target.CreateSimilar (cr.Target.Content, Width, Height);
 				
 				using (Context input_cr = new Context (dock_icon_buffer)) {
-					DrawIcons (input_cr);
+					DrawIcons (input_cr, dockArea);
 				}
 
 				int offset =  (int) (IconSize * (1 - DockIconOpacity));
@@ -241,16 +330,15 @@ namespace Docky.Interface
 			}
 		}
 		
-		void DrawIcons (Context cr)
+		void DrawIcons (Context cr, Gdk.Rectangle dockArea)
 		{
-			if (!CanFastRender) {
-				cr.AlphaFill ();
-				for (int i = 0; i < DockItems.Count; i++)
-					DrawIcon (cr, i);
-			} else {
+			bool animationRequired = AnimationRequiresRender;
+			int index = PositionProvider.IndexAtPosition (Cursor);
 			
+			if (CanFastRender && !animationRequired) {
+				// We are in a zoomed in state where we can render only those icons which are moving around
+				// on screen. This only happens when zoom is enabled
 				Gdk.Rectangle renderArea = Gdk.Rectangle.Zero;
-				Gdk.Rectangle dockArea = GetDockArea ();
 				
 				int startItemPosition;
 				startItemPosition = Math.Min (Cursor.X, RenderData.LastCursor.X) - 
@@ -263,20 +351,30 @@ namespace Docky.Interface
 				int startItem = PositionProvider.IndexAtPosition (startItemPosition, Cursor.Y);
 				int endItem = PositionProvider.IndexAtPosition (endItemPosition, Cursor.Y);
 				
+				int maxClamp = DockItems.Count - 1;
+				
+				startItem = startItem == -1 ? 0 : startItem;
+				endItem = endItem == -1 ? maxClamp : endItem;
+				
+				// these are special cases that we dont want to fall on edges (unless they are the clamps... clamps)
+				if (startItem > 0 && DockItems [startItem - 1].ContainsFocusedWindow)
+					startItem = Math.Max (0, startItem - 1);
+				
+				if (DockItems [endItem].ContainsFocusedWindow)
+					endItem = Math.Min (maxClamp, endItem + 1);
+				
 				PointD firstPosition, lastPosition;
 				double firstZoom, lastZoom;
 				
 				// set up our X value
-				if (startItem == -1) {
-					startItem = 0;
+				if (startItem == 0) {
 					renderArea.X = dockArea.X;
 				} else {
 					IconZoomedPosition (startItem, out firstPosition, out firstZoom);
 					renderArea.X = (int) (firstPosition.X - (DockItems [startItem].Width * firstZoom) / 2) - 2;
 				}
 				
-				if (endItem == -1) {
-					endItem = DockItems.Count - 1;
+				if (endItem == maxClamp) {
 					renderArea.Width = dockArea.Width - (renderArea.X - dockArea.X);
 				} else {
 					IconZoomedPosition (endItem, out lastPosition, out lastZoom);
@@ -288,38 +386,97 @@ namespace Docky.Interface
 				renderArea.Height = Height;
 				
 				cr.Rectangle (renderArea.X, renderArea.Y, renderArea.Width, renderArea.Height);
+				
+				// clear the areas outside the dock area
+				cr.Rectangle (0, dockArea.Y, dockArea.X, dockArea.Height);
+				cr.Rectangle (dockArea.X + dockArea.Width, dockArea.Y, Width - (dockArea.X + dockArea.Width), dockArea.Height);
 				switch (DockPreferences.Orientation) {
 				case DockOrientation.Bottom:
-					cr.Rectangle (0, Width, 0, Height - dockArea.Height);
+					cr.Rectangle (0, 0, Width, Height - dockArea.Height);
 					break;
 				case DockOrientation.Top:
-					cr.Rectangle (0, Width, dockArea.Height, Height - dockArea.Height);
+					cr.Rectangle (0, dockArea.Height, Width, Height - dockArea.Height);
 					break;
 				}
 				cr.Operator = Operator.Clear;
 				cr.Fill ();
 				cr.Operator = Operator.Over;
 				
-				for (int i = startItem; i <= endItem; i++)
-					DrawIcon (cr, i);
+				for (int i = startItem; i <= endItem; i++) {
+					if (i == index)
+						continue;
+					DrawIcon (cr, i, false);
+				}
+				// draw hovered item last, for the sake of making sure its on top in certain animations
+				if (index >= startItem && index <= endItem)
+					DrawIcon (cr, index, true);
+				
+			} else if (SingleItemRender && !animationRequired) {
+				// A single icon for some reason needs to be drawn again. This is more or less
+				// a special case of a fast render
+				Gdk.Rectangle renderArea = Gdk.Rectangle.Zero;
+				PointD firstPosition;
+				double firstZoom;
+				
+				AbstractDockItem single = RenderData.RenderItems [0];
+				int singleIndex = DockItems.IndexOf (single);
+				
+				// set up our X value
+				IconZoomedPosition (singleIndex, out firstPosition, out firstZoom);
+				renderArea.X = (int) (firstPosition.X - (single.Width * firstZoom) / 2) - 2;
+				renderArea.Width = (int) (firstPosition.X + (single.Width * firstZoom) / 2) + 2 - renderArea.X;
+				
+				renderArea.Height = Height;
+				
+				cr.Rectangle (renderArea.X, renderArea.Y, renderArea.Width, renderArea.Height);
+				
+				// clear the areas outside the dock area
+				cr.Rectangle (0, dockArea.Y, dockArea.X, dockArea.Height);
+				cr.Rectangle (dockArea.X + dockArea.Width, dockArea.Y, Width - (dockArea.X + dockArea.Width), dockArea.Height);
+				switch (DockPreferences.Orientation) {
+				case DockOrientation.Bottom:
+					cr.Rectangle (0, 0, Width, Height - dockArea.Height);
+					break;
+				case DockOrientation.Top:
+					cr.Rectangle (0, dockArea.Height, Width, Height - dockArea.Height);
+					break;
+				}
+				cr.Operator = Operator.Clear;
+				cr.Fill ();
+				cr.Operator = Operator.Over;
+				
+				DrawIcon (cr, singleIndex, index == singleIndex);
+			} else if (animationRequired || !CanNoRender) {
+				// we didn't fast render or single icon render, but we can't not render, so we have to do it the slow way
+				cr.AlphaFill ();
+				
+				for (int i = 0; i < DockItems.Count; i++) {
+					if (i == index)
+						continue;
+					DrawIcon (cr, i, false);
+				}
+				// draw hovered item last, for the sake of making sure its on top in certain animations
+				if (index >= 0 && index < DockItems.Count)
+					DrawIcon (cr, index, true);
 			}
 			
 			RenderData.LastCursor = Cursor;
 			RenderData.ZoomIn = ZoomIn;
 			RenderData.ForceFullRender = false;
+			RenderData.RenderItems.Clear ();
 		}
 		
-		void DrawIcon (Context cr, int icon)
+		void DrawIcon (Context cr, int icon, bool hovered)
 		{
 			// Don't draw the icon we are dragging around
-			if (GtkDragging && !DragState.IsFinished) {
-				int item = DockItems.IndexOf (DragState.DragItem);
+			if (DnDTracker.GtkDragging && !DnDTracker.DragState.IsFinished) {
+				int item = DockItems.IndexOf (DnDTracker.DragState.DragItem);
 				if (item == icon && DockServices.ItemsService.ItemCanBeMoved (item))
 					return;
 			}
 			
 			AbstractDockItem dockItem = DockItems [icon];
-			if (dockItem == null) return;
+			if (dockItem == null) return; //happens?
 			
 			PointD center;
 			double zoom;
@@ -334,7 +491,7 @@ namespace Docky.Interface
 			// we will set this flag now
 			if (animationType == ClickAnimationType.Bounce) {
 				// bounces twice
-				double delta = Math.Abs (30 * Math.Sin 
+				double delta = Math.Abs (LaunchBounceHeight * Math.Sin 
 				                         (dockItem.TimeSinceClick.TotalMilliseconds * Math.PI / 
 				                          (BounceTime.TotalMilliseconds / 2)));
 				
@@ -344,7 +501,7 @@ namespace Docky.Interface
 					double urgentMs = (RenderTime - dockItem.AttentionRequestStartTime)
 						.TotalMilliseconds;
 					
-					double delta = 100 * Math.Sin (urgentMs * Math.PI / (BounceTime.TotalMilliseconds));
+					double delta = UrgentBounceHeight * Math.Sin (urgentMs * Math.PI / (BounceTime.TotalMilliseconds));
 					iconPosition = iconPosition.RelativeMovePoint (delta, RelativeMove.Inward);
 				}
 			}
@@ -364,6 +521,44 @@ namespace Docky.Interface
 					return;
 				}
 				
+				if (dockItem.ContainsFocusedWindow) {
+					double intenseS = 0.8;
+					
+					double xHigh = iconPosition.X - 1.5;
+					double yHigh = MinimumDockArea.Y;
+					double widthHigh = dockItem.Width * zoom + 3;
+					cr.Rectangle (xHigh, yHigh, widthHigh, DockHeight);
+					
+					LinearGradient lg;
+					if (DockPreferences.Orientation == DockOrientation.Bottom) {
+						lg = new LinearGradient (0, yHigh + DockHeight, 0, yHigh);
+					} else {
+						lg = new LinearGradient (0, yHigh, 0, yHigh + DockHeight);
+					}
+					
+					Cairo.Color average = dockItem.AverageColor ();
+					lg.AddColorStop (0, new Cairo.Color (average.R, average.G, average.B, .3));
+					lg.AddColorStop (1, new Cairo.Color (average.R, average.G, average.B, 0));
+					cr.Pattern = lg;
+					cr.Fill ();
+					lg.Destroy ();
+					
+					if (DockPreferences.Orientation == DockOrientation.Bottom)
+						cr.Translate (0, 2);
+					
+					cr.MoveTo (xHigh, yHigh);
+					cr.LineTo (xHigh, yHigh + DockHeight - 2);
+					cr.MoveTo (xHigh + widthHigh, yHigh);
+					cr.LineTo (xHigh + widthHigh, yHigh + DockHeight - 2);
+					
+					if (DockPreferences.Orientation == DockOrientation.Bottom)
+						cr.Translate (0, -2);
+					
+					cr.Color = new Cairo.Color (intenseS, intenseS, intenseS, .3);
+					cr.LineWidth = 1;
+					cr.Stroke ();
+				}
+				
 				if (scale != 1)
 					cr.Scale (scale, scale);
 				// we need to multiply x and y by 1 / scale to undo the scaling of the context.  We only want to zoom
@@ -373,9 +568,9 @@ namespace Docky.Interface
 				                                 InsertAnimationTime.TotalMilliseconds, 1);
 				cr.SetSource (iconSurface, 
 				              iconPosition.X / scale, iconPosition.Y / scale);
-				cr.PaintWithAlpha (fadeInOpacity);
+				cr.PaintWithAlpha (fadeInOpacity * (dockItem.ContainsMinimizedWindow ? .5 : 1));
 				
-				bool shade_light = GtkDragging && 
+				bool shade_light = DnDTracker.GtkDragging && !DnDTracker.PreviewIsDesktopFile && CursorIsOverDockArea &&
 					dockItem.IsAcceptingDrops && icon == PositionProvider.IndexAtPosition (Cursor);
 				
 				bool shade_dark = animationType == ClickAnimationType.Darken;
@@ -424,18 +619,21 @@ namespace Docky.Interface
 			// we do a null check here to allow things like separator items to supply
 			// a null.  This allows us to draw nothing at all instead of rendering a
 			// blank surface (which is slow)
-			if (!PopupMenu.Visible && PositionProvider.IndexAtPosition (Cursor) == icon &&
-			    CursorIsOverDockArea && dockItem.GetTextSurface (cr.Target) != null && !GtkDragging) {
+			if (!PopupMenu.Visible && hovered &&
+			    CursorIsOverDockArea && dockItem.GetTextSurface (cr.Target) != null && 
+			    !DnDTracker.GtkDragging && !DnDTracker.DragResizing) {
 
 				Gdk.Point textPoint;
-				textPoint.X = PositionProvider.IconUnzoomedPosition (icon).X - (DockPreferences.TextWidth >> 1);
+				Surface textSurface = dockItem.GetTextSurface (cr.Target);
+				textPoint.X = PositionProvider.IconUnzoomedPosition (icon).X - (dockItem.TextSurfaceSize.Width >> 1);
+				textPoint.X = Math.Max (0, Math.Min (Width - dockItem.TextSurfaceSize.Width, textPoint.X));
 				
 				if (DockPreferences.Orientation == DockOrientation.Top)
 					textPoint.Y = (int) (DockPreferences.ZoomPercent * IconSize) + 10;
 				else
 					textPoint.Y = Height - (int) (DockPreferences.ZoomPercent * IconSize) - 38;
 				
-				dockItem.GetTextSurface (cr.Target).Show (cr, textPoint.X, textPoint.Y);
+				textSurface.Show (cr, textPoint.X, textPoint.Y);
 			}
 		}
 		
@@ -471,31 +669,32 @@ namespace Docky.Interface
 			else
 				rect = PositionProvider.DockArea (ZoomIn, Cursor);
 
-			int minSize; 
+			int minWidth = 10 * rect.Height;
+			int maxWidth = LayoutUtils.MonitorGeometry ().Width;
+			int dockWidth = 0;
 				
 			if (PainterOverlayVisible && Painter != null) {
-				minSize = Math.Max (Painter.MinimumWidth, 10 * rect.Height);
+				dockWidth = Math.Min (Painter.Width, maxWidth);
 			} else if (!PainterOverlayVisible && LastPainter != null) {
-				minSize = Math.Max (LastPainter.MinimumWidth, 10 * rect.Height);
+				dockWidth = Math.Min (LastPainter.Width, maxWidth);
 			} else {
-				minSize = 10 * rect.Height;
+				dockWidth = Math.Max (rect.Width, minWidth);
 			}
 			
-			if (rect.Width < minSize && DockIconOpacity < 1) {
-				int difference = minSize - rect.Width;
+			if (rect.Width != dockWidth && DockIconOpacity < 1) {
+				int difference = dockWidth - rect.Width;
 				int alpha = (int) (difference * PainterOpacity);
 				rect.X -= alpha / 2;
 				rect.Width += alpha;
 			}
 			
-			
 			return rect;
 		}
 		
 		Surface GetIndicator (Surface similar)
 		{
 			if (indicator == null) {
-				Style style = Docky.Interface.DockWindow.Window.Style;
+				Style style = window.Style;
 				Gdk.Color color = style.Backgrounds [(int) StateType.Selected].SetMinimumValue (100);
 
 				indicator = similar.CreateSimilar (similar.Content, IndicatorSize * 2, IndicatorSize * 2);
@@ -598,11 +797,12 @@ namespace Docky.Interface
 
 		protected override bool OnExposeEvent(EventExpose evnt)
 		{
-			if (!IsDrawable || window.IsRepositionHidden)
+			if (!IsDrawable)
 				return false;
 			
+			rendering = true;
+			zoom_in_buffer = null;
 			RenderTime = DateTime.UtcNow;
-			
 			Context cr;
 			if (backbuffer == null) {
 				cr = Gdk.CairoHelper.Create (GdkWindow);
@@ -621,13 +821,15 @@ namespace Docky.Interface
 					FirstRenderTime = DateTime.UtcNow;
 					first_render_set = true;
 				}
-				DrawDrock (cr);
+				DrawDock (cr);
 			}
 			(cr as IDisposable).Dispose ();
 			
 			cr = Gdk.CairoHelper.Create (GdkWindow);
 			
-			Gdk.Point finalTarget = new Gdk.Point (0, 0).RelativeMovePoint (VerticalOffset, RelativeMove.Outward);
+			int vert = VerticalOffset;
+			Gdk.Point finalTarget = new Gdk.Point (0, 0).RelativeMovePoint (vert, RelativeMove.Outward);
+			
 			cr.SetSource (backbuffer, finalTarget.X, finalTarget.Y);
 			
 			cr.Operator = Operator.Source;
@@ -637,6 +839,7 @@ namespace Docky.Interface
 			((IDisposable)cr.Target).Dispose ();
 			((IDisposable)cr).Dispose ();
 			
+			rendering = false;
 			return true;
 		}
 		
@@ -657,9 +860,18 @@ namespace Docky.Interface
 			base.OnStyleSet (previous_style);
 		}
 		
+		void RequestIconRender (AbstractDockItem item)
+		{
+			if (RenderData != null) {
+				RenderData.RenderItems.Add (item);
+			}
+		}
+		
 		void RequestFullRender ()
 		{
-			RenderData.ForceFullRender = true;
+			if (RenderData != null) {
+				RenderData.ForceFullRender = true;
+			}
 		}
 		
 		void ResetBuffers()
@@ -679,10 +891,12 @@ namespace Docky.Interface
 				input_area_buffer = null;
 			}
 			
-			foreach (Surface sr in painter_surfaces.Values) {
-				sr.Destroy ();
+			if (painter_surfaces != null) {
+				foreach (Surface sr in painter_surfaces.Values) {
+					sr.Destroy ();
+				}
+				painter_surfaces.Clear ();
 			}
-			painter_surfaces.Clear ();
 			
 			RequestFullRender ();
 		}
