@@ -52,9 +52,9 @@ namespace Do
 {
 	internal static class AssemblyInfo
 	{
-		public const string DisplayVersion = "0.8.1.3";
-		public const string VersionDetails = "bzr do r1102";
-		public const string LocaleDirectory = "/usr/share/locale";
+		public const string DisplayVersion = "0.8.2";
+		public const string VersionDetails = "bzr trunk r1261";
+		public const string LocaleDirectory = "/usr/local/share/locale";
 	}
 }
 
