@@ -48,12 +48,11 @@ namespace Do.UI
 			base (WindowType.Toplevel)
 		{
 			Build ();
+			IconName = "gnome-desktop-config";
 
 			btn_close.IsFocus = true;
 			
-			TargetEntry [] targets = {
-				new TargetEntry ("text/uri-list", 0, 0), 
-			};
+			TargetEntry [] targets = { new TargetEntry ("text/uri-list", 0, 0) };
 			Drag.DestSet (this, DestDefaults.All, targets, Gdk.DragAction.Copy);
 			
 			// Add notebook pages.
@@ -74,5 +73,12 @@ namespace Do.UI
 		{
 			Services.Environment.OpenUrl (HelpUrl);
 		}
+		
+		public override void Dispose ()
+		{
+			foreach (Gtk.Widget w in notebook.Children)
+				w.Dispose ();
+			base.Dispose ();
+		}
 	}
 }
